"use strict";

exports.__esModule = true;
exports.default = createContext;

var _hooks = require("./hooks");

var _types = require("./types");

var ValueEmitter =
/*#__PURE__*/
function () {
  function ValueEmitter(defaultValue) {
    this.handlers = [];
    this.value = defaultValue;
  }

  var _proto = ValueEmitter.prototype;

  _proto.on = function on(handler) {
    this.handlers.push(handler);
  };

  _proto.off = function off(handler) {
    this.handlers = this.handlers.filter(function (h) {
      return h !== handler;
    });
  };

  _proto.emit = function emit() {
    var _this = this;

    this.handlers.forEach(function (handler) {
      return handler(_this.value);
    });
  };

  return ValueEmitter;
}();

var uniqueId = 0;

function createContext(defaultValue) {
  var contextProp = '__ctx' + uniqueId++;
  var stack = [];
  var defaultEmitter = new ValueEmitter(defaultValue);

  function Provider(props) {
    var propsValue = props.value !== undefined ? props.value : defaultValue;

    var _useState = (0, _hooks.useState)(propsValue),
        value = _useState[0],
        setValue = _useState[1];

    var _useState2 = (0, _hooks.useState)(function () {
      return new ValueEmitter(value);
    }),
        emitter = _useState2[0];

    emitter.value = propsValue;
    if (propsValue !== value) setValue(propsValue);
    (0, _hooks.useEffect)(function () {
      stack.pop();
    });
    (0, _hooks.useEffect)(function () {
      emitter.emit();
    }, [value]);
    stack.push(emitter);
    return props.children;
  }

  function readEmitter(instance) {
    var emitter = stack[stack.length - 1];
    if (emitter) return emitter;

    while (instance && instance._internal) {
      if (instance instanceof Provider) {
        break;
      }

      instance = instance._internal._parentInstance;
    }

    return instance && instance.emitter || defaultEmitter;
  }

  Provider.readEmitter = readEmitter;
  Provider.contextProp = contextProp;

  function Consumer(props) {
    var _this2 = this;

    var _useState3 = (0, _hooks.useState)(function () {
      return readEmitter(_this2);
    }),
        emitter = _useState3[0];

    var _useState4 = (0, _hooks.useState)(emitter.value),
        value = _useState4[0],
        setValue = _useState4[1];

    if (value !== emitter.value) {
      setValue(emitter.value);
      return; // Interrupt execution of consumer.
    }

    function onUpdate(updatedValue) {
      if (value !== updatedValue) {
        setValue(updatedValue);
      }
    }

    (0, _hooks.useEffect)(function () {
      emitter.on(onUpdate);
      return function () {
        emitter.off(onUpdate);
      };
    }, []);
    var children = props.children;
    var consumer = (0, _types.isArray)(children) ? children[0] : children;

    if ((0, _types.isFunction)(consumer)) {
      return consumer(value);
    }
  }

  return {
    Provider: Provider,
    Consumer: Consumer
  };
}